async function generateReducedColorSVG(image, colorCount = 8, svgWidth = 32) {
	const canvas = document.createElement('canvas')
	const ctx = canvas.getContext('2d')

	const aspectRatio = image.naturalHeight / image.naturalWidth
	canvas.width = svgWidth
	canvas.height = svgWidth * aspectRatio

	ctx.drawImage(image, 0, 0, canvas.width, canvas.height)

	const imageData = ctx.getImageData(0, 0, canvas.width, canvas.height)
	const pixels = imageData.data

	const clusters = kMeans(pixels, colorCount)
	const reducedPixels = assignToClusters(pixels, clusters)

	const svg = generateSVGFromPixels(reducedPixels, canvas.width, canvas.height)
	sendPackImageReduction(svg)
}

function kMeans(data, k) {
	const clusters = Array.from({length: k}, () => ({
		r: Math.random() * 255,
		g: Math.random() * 255,
		b: Math.random() * 255,
	}))

	let assignments = new Array(data.length / 4).fill(0)
	let changed = true

	while (changed) {
		changed = false

		for (let i = 0; i < data.length; i += 4) {
			const r = data[i]
			const g = data[i + 1]
			const b = data[i + 2]

			let minDistance = Infinity
			let clusterIndex = 0

			clusters.forEach((cluster, idx) => {
				const distance =
					Math.pow(r - cluster.r, 2) +
					Math.pow(g - cluster.g, 2) +
					Math.pow(b - cluster.b, 2)

				if (distance < minDistance) {
					minDistance = distance
					clusterIndex = idx
				}
			})

			if (assignments[i / 4] !== clusterIndex) {
				assignments[i / 4] = clusterIndex
				changed = true
			}
		}

		const newClusters = clusters.map(() => ({r: 0, g: 0, b: 0, count: 0}))

		for (let i = 0; i < data.length; i += 4) {
			const clusterIndex = assignments[i / 4]
			newClusters[clusterIndex].r += data[i]
			newClusters[clusterIndex].g += data[i + 1]
			newClusters[clusterIndex].b += data[i + 2]
			newClusters[clusterIndex].count += 1
		}

		clusters.forEach((cluster, idx) => {
			if (newClusters[idx].count > 0) {
				cluster.r = newClusters[idx].r / newClusters[idx].count
				cluster.g = newClusters[idx].g / newClusters[idx].count
				cluster.b = newClusters[idx].b / newClusters[idx].count
			}
		})
	}

	return clusters
}

function assignToClusters(data, clusters) {
	const reducedPixels = new Uint8ClampedArray(data.length)

	for (let i = 0; i < data.length; i += 4) {
		const r = data[i]
		const g = data[i + 1]
		const b = data[i + 2]

		let minDistance = Infinity
		let clusterIndex = 0

		clusters.forEach((cluster, idx) => {
			const distance =
				Math.pow(r - cluster.r, 2) + Math.pow(g - cluster.g, 2) + Math.pow(b - cluster.b, 2)

			if (distance < minDistance) {
				minDistance = distance
				clusterIndex = idx
			}
		})

		reducedPixels[i] = clusters[clusterIndex].r
		reducedPixels[i + 1] = clusters[clusterIndex].g
		reducedPixels[i + 2] = clusters[clusterIndex].b
		reducedPixels[i + 3] = 25
	}

	return reducedPixels
}

function generateSVGFromPixels(pixels, width, height) {
	const svg = ['<svg xmlns="http://www.w3.org/2000/svg" ', `width="${width}" height="${height}">`]
	const rectSize = 1

	for (let y = 0; y < height; y++) {
		for (let x = 0; x < width; x++) {
			const idx = (y * width + x) * 4
			const r = pixels[idx]
			const g = pixels[idx + 1]
			const b = pixels[idx + 2]
			const color = `rgb(${r},${g},${b})`

			svg.push(
				`<rect x="${x}" y="${y}" width="${rectSize}" height="${rectSize}" fill="${color}" />`
			)
		}
	}

	svg.push('</svg>')
	return svg.join('')
}
