const TerminalError = {
	MissingArgument: 'Missing argument (no path to pack passed)',
	NoPackDirectory: 'No pack directory found on disk',
}

const Errors = {
	MissingPackJSON: 'Missing Pack JSON',
	MissingPadsJSON: 'Missing Pads JSON',
	MissingPreview: 'Missing Preview',

	PreviewNotValid: 'Preview isnt valid',

	PackJSONInvalid: 'Pack JSON is invalid or unreadable',
	PackJSONNameInvalid: 'Pack JSON Name invalid or empty',
	PackJSONUUIDInvalid: 'Pack JSON UUID invalid or empty',
	PackJSONDescriptionInvalid: 'Pack JSON Description invalid or empty',
	PackJSONShortDescriptionInvalid: 'Pack JSON Short Description invalid or empty',
	PackJSONVersionNotAsExpected: 'Pack JSON Version not 1',
	PackJSONAudioPreviewInvalid: 'Pack JSON Audio Preview invalid or empty',
	PackJSONImageInvalid: 'Pack JSON Image invalid or empty',
	PackJSONFreeToyInvalid: 'Pack JSON Free Toy invalid',
	PackJSONAuthorInvalid: 'Pack JSON Author invalid or empty',
	PackJSONAuthorLinkInvalid: 'Pack JSON Author Link invalid or empty',
	PackJSONPreviewLinkInvalid: 'Pack JSON Preview Link invalid or empty',
	PackJSONDateInvalid: 'Pack JSON Date invalid or empty',
	PackJSONTagsInvalid: 'Pack JSON Tags invalid or empty',

	PadsJSONInvalid: 'Pad JSON is invalid or unreadable',
	PadsJSONNoKits: 'Pad JSON has no kits',
	PadWithMissingID: 'Pad has missing ID',
	InvalidPadIDs: 'Invalid Pad IDs',
}

const packErrors = []

function testForMissingFiles(data) {
	if (data.packJSON === 'MISSING') {
		packErrors.push(Errors.MissingPackJSON)
	}
	if (data.padsJSON === 'MISSING') {
		packErrors.push(Errors.MissingPadsJSON)
	}
	if (data.preview === 'MISSING') {
		packErrors.push(Errors.MissingPreview)
	}
}

function testPackJSONIsValid(json) {
	const validations = [
		{
			field: 'name',
			check: (value) => value && value.trim(),
			errorCode: Errors.PackJSONNameInvalid,
		},
		{
			field: 'uuid',
			check: (value) =>
				/^[0-9a-fA-F]{8}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{12}$/.test(
					value
				),
			errorCode: Errors.PackJSONUUIDInvalid,
		},
		{
			field: 'description',
			check: (value) => value && value.trim(),
			errorCode: Errors.PackJSONDescriptionInvalid,
		},
		{
			field: 'shortDescription',
			check: (value) => value && value.trim(),
			errorCode: Errors.PackJSONShortDescriptionInvalid,
		},
		{
			field: 'version',
			check: (value) => value === '1',
			errorCode: Errors.PackJSONVersionNotAsExpected,
		},
		{
			field: 'audioPreview',
			check: (value) => value && value.trim(),
			errorCode: Errors.PackJSONAudioPreviewInvalid,
		},
		{
			field: 'freeToy',
			check: (value) => value !== null,
			errorCode: Errors.PackJSONFreeToyInvalid,
		},
		{
			field: 'image',
			check: (value) => value && value.trim(),
			errorCode: Errors.PackJSONImageInvalid,
		},
		{
			field: 'smallImage',
			check: (value) => value && value.trim(),
			errorCode: Errors.MissingSmallImage,
		},
		{
			field: 'authorLink',
			check: (value) => value && value.trim(),
			errorCode: Errors.PackJSONAuthorLinkInvalid,
		},
		{
			field: 'previewLink',
			check: (value) => value && value.trim(),
			errorCode: Errors.PackJSONPreviewLinkInvalid,
		},
		{
			field: 'imageLink',
			check: (value) => value && value.trim(),
			errorCode: Errors.PackJSONImageLinkInvalid,
		},
		{
			field: 'author',
			check: (value) => value && value.trim(),
			errorCode: Errors.PackJSONAuthorInvalid,
		},
		{
			field: 'tags',
			check: (value) => Array.isArray(value) && value.length > 0,
			errorCode: Errors.PackJSONTagsInvalid,
		},
		{
			field: 'versionDateUtc',
			check: (value) => value && value.trim(),
			errorCode: Errors.PackJSONDateInvalid,
		},
	]

	for (const {field, check, errorCode} of validations) {
		if (!check(json[field])) {
			packErrors.push(errorCode)
		}
	}
}

function testAllPadsHaveIDs(pads) {
	if (pads.filter((pad) => !pad.sampleId).length > 0) {
		packErrors.push(Errors.PadWithMissingID)
	}
}

function testAllKitsHaveValidPadIDs(pads, kits) {
	if (kits === null || kits === undefined) {
		packErrors.push(Errors.PadsJSONNoKits)
		return
	}

	kits.forEach((kit) => {
		if (kit.padIds.length !== 16) {
			packErrors.push(Errors.InvalidPadIDs)
		}
	})
}

function testForUnusedSamples(pads, kits) {
	if (!kits || !Array.isArray(kits)) {
		return
	}

	const unusedPads = pads
		.filter((pad) => !kits.some((kit) => (kit.padIds || []).includes(pad.id)))
		.map((pad) => pad.id)

	if (unusedPads.length > 0) {
		console.log(`Unused pads: ${unusedPads}`)
	}
}

function testPadsJSONIsValid(json) {
	testAllPadsHaveIDs(json.pads || [])
	testAllKitsHaveValidPadIDs(json.pads || [], json.kits || null)
}

function testImageIsValid(json) {
	const rawData = json
	if (rawData.length < 2) {
		packErrors.push(Errors.ImageNotValid)
		return
	}

	const isPNG =
		rawData[0] === 0x89 &&
		rawData[1] === 0x50 &&
		rawData[2] === 0x4e &&
		rawData[3] === 0x47 &&
		rawData[4] === 0x0d &&
		rawData[5] === 0x0a &&
		rawData[6] === 0x1a &&
		rawData[7] === 0x0a
	const isJPG =
		rawData[0] === 0xff &&
		rawData[1] === 0xd8 &&
		rawData[rawData.length - 2] === 0xff &&
		rawData[rawData.length - 1] === 0xd9

	if (isPNG) {
		return
	}

	if (isJPG) {
		packErrors.push(Errors.ImageIsJPGNotPNG)
		return
	}
	packErrors.push(Errors.ImageNotValid)
}

function testPreviewIsValid(json) {
	const rawData = json
	if (rawData.length < 2) {
		packErrors.push(Errors.PreviewNotValid)
		return
	}
	if (rawData[0] === 0xff && (rawData[1] & 0xe0) === 0xe0) {
	} else {
		packErrors.push(Errors.PreviewNotValid)
	}
}

function printErrors(packName) {
	const row = Array.from(document.querySelector('#pack-table-list').rows).find(
		(row) => row.dataset.id === packName
	)

	if (row) {
		const cells = row.cells
		cells[0].innerHTML =
			packErrors.length === 0
				? '<span style="font-size: 1.25em; color: green; display: inline-block;">✓</span>'
				: '<span style="font-size: 0.75em; color: red; display: inline-block;">❌</span>'

		const firstError = packErrors[0]
		if (packErrors.length > 1) {
			cells[2].textContent = `${firstError} + ${packErrors.length - 1} more`
		} else {
			cells[2].textContent = firstError
		}
		cells[2].setAttribute('title', packErrors.join('\n'))
	} else {
		console.error(`Row with data-id="${packName}" not found`)
	}
}

function testPack(data) {
	packErrors.length = 0
	if ('error' in data) {
		packErrors.push('Missing Pack')
		printErrors(data.packName)
		return
	}

	testForMissingFiles(data)
	if (data.packJSON !== 'MISSING') {
		testPackJSONIsValid(data.packJSON)
	}
	if (data.padsJSON !== 'MISSING') {
		testPadsJSONIsValid(data.padsJSON)
	}
	testImageIsValid(data.image || [])
	testPreviewIsValid(data.preview || [])
	printErrors(data.packName)
}
